/*
 * Copyright (C) 2005 Andre Noll <maan@systemlinux.org>
 *
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 */


#include <openssl/pem.h>
#include "para.h"

#define LOAD_PUBLIC_KEY 0
#define LOAD_PRIVATE_KEY 1

static EVP_PKEY *load_key(const char *file, int private)
{
	BIO *key;
	EVP_PKEY *pkey = NULL;

	key = BIO_new(BIO_s_file());
	if (!key)
		return NULL;
	if (BIO_read_filename(key, file) > 0)
		pkey = private?
			PEM_read_bio_PrivateKey(key, NULL, NULL, NULL) :
			PEM_read_bio_PUBKEY(key, NULL, NULL, NULL);
	BIO_free(key);
	return pkey;
}

static int get_key(char *key_file, RSA **rsa, int private)
{
	EVP_PKEY *pkey;

	pkey = load_key(key_file, private);
	if (!pkey)
		return -1; /* can not read private key */
	*rsa = EVP_PKEY_get1_RSA(pkey);
        EVP_PKEY_free(pkey);
        if (!*rsa)
		return -2; /* rsa error */
	return RSA_size(*rsa);
}

int para_decrypt(char *key_file, long unsigned *challenge_nr, unsigned char *buf, int rsa_inlen)
{
	RSA *rsa;
	unsigned char *rsa_out;
	int ret, rsa_outlen, keysize = get_key(key_file, &rsa, LOAD_PRIVATE_KEY);

	if (keysize < 0)
		return -1;
        rsa_out = OPENSSL_malloc(keysize);
	rsa_outlen = RSA_private_decrypt(rsa_inlen, buf, rsa_out, rsa, RSA_PKCS1_PADDING);
	ret = -2;
	if (rsa_outlen <= 0)
		goto out; /* decrypt error */
	ret = sscanf((char *)rsa_out, "%lu", challenge_nr) == 1? 1: -3;
out:
	OPENSSL_free(rsa_out);
	return ret;
}

int para_encrypt(char *key_file, long unsigned challenge_nr, unsigned char *outbuf)
{
	RSA *rsa;
	unsigned char *inbuf = (unsigned char*) make_message("%lu", challenge_nr);
	int outlen, ret, keysize = get_key(key_file, &rsa, LOAD_PUBLIC_KEY);

	ret = keysize;
	if (keysize < 0)
		goto out;
	ret = -2;
	outlen = RSA_public_encrypt(strlen((char *)inbuf), inbuf, outbuf, rsa, RSA_PKCS1_PADDING);
	if (outlen <= 0)
		goto out; /* encrypt error */
	ret = outlen;
out:
	free(inbuf);
	return ret;
}
